"use strict";
module.exports = Cache;

var createHash = require('crypto').createHash;

function Cache() {
	this._cache = {};
}

Cache.prototype.get = function get(sql, expires) {
	var key = this.key(sql);
	if (this._cache.hasOwnProperty(key) && this._cache[key]) {
		//	if (!expires || ((Date.now() - cache.start) < expires)) {
		return this._cache[key];
		//	}
	}
	return false;
};

Cache.prototype.create = function (sql, expires) {
	return new Item(this, sql, expires);
};

Cache.prototype.key = function (sql) {
	return createHash('md5').update(sql).digest('base64');
};

Cache.prototype.add = function add(c) {
	this._cache[c.key] = c;
};

Cache.prototype.del = function add(c) {
	delete this._cache[c.key];
};


function Item(mgr, sql, expires) {
	this.mgr = mgr;
	this.sql = sql;
	this.key = mgr.key(sql);
	this.start = Date.now();
	this.expires = expires;
	this.cb = [];
	this.mgr.add(this);
	if (this.expires) {
		this.tmr = setTimeout(mgr.del.bind(mgr, this), this.expires * 1000);
	}
	this.result = null;
	this.err = null;
	this.done = false;
}

Item.prototype.add = function add(cb) {
	if (this.done) {
		cb(this.err, this.result);
	} else {
		this.cb.push(cb);
	}
};

Item.prototype.doneF = function done(err, result) {
	this.done = true;
	this.result = result;
	this.err = err;
	if (!this.expires) {
		this.mgr.del(this);
	}
	var cb;
	while (cb = this.cb.pop()) {
		process.nextTick(function(){
			this(err, result);
		}.bind(cb));
	}
};


