"use strict";
module.exports = Delete;
var Where = require('./where.js');

/**
 * Database query builder for DELETE statements. See {Builder} for usage and examples.
 *
 * @implements {Where}
 * @param {DB} db
 * @param {string} table
 * @constructor
 */
function Delete(db, table) {
	Where.call(this, db, db.DELETE);
	this._table = table || null;
}

Delete.inherits(Where);
 
/**
 * Sets the table to delete from.
 *
 * @param {string}  table name or array($table, $alias) or object
 * @return  {Delete}
 */
Delete.prototype.table = function table(table) {
	this._table = table;
	return this;
};

/**
 * Compile the SQL query and return it.
 *
 * @return  {string}
 */
Delete.prototype.compile = function compile() {
	// Start a deletion query
	var query = 'DELETE FROM ' + this.db.quoteTable(this._table);

	if (this._where.length > 0) {
		// Add deletion conditions
		query += ' WHERE ' + this._compileConditions(this._where);
	}

	if (this._orderBy.length) {
		// Add sorting
		query += ' ' + this._compileOrderBy(this._orderBy);
	}

	if (this._limit !== null) {
		// Add limiting
		query += ' LIMIT ' + this._limit;
	}

	this._sql = query;

	return Where.prototype.compile.call(this);
};

/**
 * Reset table name
 * @return {Delete}
 */
Delete.prototype.reset = function reset() {
	this._table = null;
	return Where.prototype.reset.call(this);
};