"use strict";
module.exports = Result;

/**
 * DB Result
 *
 * Implements `forEach`, `map`, `filter`, `slice`, `reverse`, `sort` of Array
 *
 * @param {String} sql Current sql
 * @param {Array} result Rows as array
 * @constructor
 */
function Result(sql, result, fields) {
	Object.defineProperties(this, {
		length : {
			value      : result.length,
			writeable  : false,
			enumerable : true},
		sql    : {
			value      : sql,
			writeable  : false,
			enumerable : true},
		fields : {
			value      : fields,
			writeable  : false,
			enumerable : true
		}
	});

	result.forEach(function (row, index) {
		Object.defineProperty(this, index, {
			value      : row,
			writeable  : false,
			enumerable : true
		});
	}, this);
}

/**
 * Convert rows to custom object
 *
 * @param {function} func Function iterated in Array.map
 * @param thisArg
 * @return {Result}
 */
Result.prototype.toObject = function toObject(func, thisArg) {
	var result = this.map(func, thisArg);
	Object.defineProperties(result, {
		sql : {
			value      : this.sql,
			writeable  : false,
			enumerable : true}
	});
	return result;
};

/**
 * Convert rows to indexed arrays
 *
 * @param {function} func
 * @param thisArg
 * @return {Result}
 */
Result.prototype.toArray = function toObject(func, thisArg) {
	return this.toObject(function (row) {
		return Object.keys(row).map(function (field) {
			return this[field];
		}, row)
	});
};

/**
 * Rows count
 *
 * @type {Number}
 */
Result.prototype.length = 0;

/**
 * Current SQL
 *
 * @type {String}
 */
Result.prototype.sql = 0;

['forEach', 'map', 'filter', 'slice', 'reverse', 'sort'].forEach(function (method) {
	this[method] = Array.prototype[method];
}, Result.prototype);

Result.prototype.forAll = function forAll(method, end) {
	if (this.length == 0) {
		end(false, this);
	} else {
		var c = this.length;
		this.forEach(function (item, idx) {
			method(item, idx, function (err) {
				if (err) {
					end(err);
				} else if (--c == 0) {
					end(false, this);
				}
			}.bind(this));
		}.bind(this));
	}
};
